'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = Characteristic;
function Characteristic(uuid, Parent, util, descriptor, _ref) {
  var encode = _ref.encode,
      decode = _ref.decode;

  function ReduxCharacteristic() {
    Parent.call(this, {
      uuid: uuid,
      properties: ['writeWithoutResponse', 'read', 'notify'],
      descriptors: [descriptor]
    });
    this.messages = {};
    this.maxValueSize = 20;
  }

  util.inherits(ReduxCharacteristic, Parent);

  ReduxCharacteristic.prototype.onWriteRequest = function (data, offset, withoutResponse, callback) {
    if (offset) {
      callback(this.RESULT_ATTR_NOT_LONG);
      return this.RESULT_ATTR_NOT_LONG;
    }

    var buffer = decode(data);
    var id = buffer.slice(0, 9);
    var chunk = buffer.slice(10, this.maxValueSize);
    var message = (this.messages[id] || '') + chunk;
    if (message.startsWith('[[[') && message.endsWith(']]]')) {
      var action = JSON.parse(message.slice(3, message.length - 3));
      this.onAction(action);
      this.messages[id] = '';
    } else {
      this.messages[id] = message;
    }

    callback(this.RESULT_SUCCESS);
    return this.RESULT_SUCCESS;
  };

  ReduxCharacteristic.prototype.onReadRequest = function (offset, callback) {
    if (offset) {
      callback(this.RESULT_ATTR_NOT_LONG, null);
      return;
    }
    var configuration = encode('|||' + JSON.stringify(this.configuration) + '|||');
    callback(this.RESULT_SUCCESS, configuration);
  };

  ReduxCharacteristic.prototype.onSubscribe = function (maxValueSize, updateValueCallback) {
    this.maxValueSize = maxValueSize;
    this.updateValueCallback = updateValueCallback;
    this.configuration = {
      limit: maxValueSize
    };
  };

  ReduxCharacteristic.prototype.onUnsubscribe = function () {
    this.updateValueCallback = null;
  };

  ReduxCharacteristic.prototype.onAction = function () {
    return true;
  };

  ReduxCharacteristic.prototype.updateState = function (state) {
    if (this.updateValueCallback) {
      var message = encode('[[[' + JSON.stringify(state) + ']]]');
      var i = 0;
      do {
        var next = i + this.maxValueSize;
        var end = Math.min(next, message.length);
        var data = message.slice(i, end);
        this.updateValueCallback(data);
        i = next;
      } while (i < message.length);
    }
  };

  return new ReduxCharacteristic();
}